/*

*/
package scene;

/**
<pre>
   This is a (non-singular) orthographic projection matrix.
     [ 1  0  0  0 ]
     [ 0  1  0  0 ]
     [ 0  0  1  1 ]
     [ 0  0  0  1 ]
   Notice that this matrix maps
        (x, y, z, 1) -> (x, y, z+1, 1).
   This matrix transforms the standard view parallelepiped
   (in camera coordinates) into the standard view parallelepiped
   (in clip coordinates, which are also NDC in this case) and it
   translates the standard view plane from z = -1 (in camera
   coordinates) to z = 0 (in NDC).

   But we want the near plane z = -near in camera coordinates to
   be translated to the clipping plane at z = +1 in NDC. So let us
   change the translation factor in the orthographic projection matrix,
     [ 1  0  0  0 ]
     [ 0  1  0  0 ]
     [ 0  0  1  b ]
     [ 0  0  0  1 ]
   so that z = -near gets transformed to z = +1. The new matrix maps
        (x, y, -near, 1) -> (x, y, -near + b, 1)
   and we want
              -near + b = 1
   so
               b = 1 + near.
</pre>
*/
public class OrthographicProjectionMatrix1 extends Matrix
{
   /**
   <pre>
      Construct this orthographic projection matrix.
        [ 1  0  0   0   ]
        [ 0  1  0   0   ]
        [ 0  0  1 1+near]
        [ 0  0  0   1   ]
   </pre>
   */
   public OrthographicProjectionMatrix1(double n)
   {
      super(  new Vector(1.0, 0.0, 0.0, 0.0),
              new Vector(0.0, 1.0, 0.0, 0.0),
              new Vector(0.0, 0.0, 1.0, 0.0),
              new Vector(0.0, 0.0, 1+n, 1.0) );
   }
}
